/********************************************************
 * Arduino PID Tuning Front-End,  Version 0.3
 * by Brett Beauregard
 * License: Creative-Commons Attribution Share-Alike
 * April 2011
 *
 * This application is designed to interface with an
 * arduino running the PID Library.  From this Control
 * Panel you can observe & adjust PID performance in 
 * real time
 *
 * The ControlP5 library is required to run this sketch.
 * files and install instructions can be found at
 * http://www.sojamo.de/libraries/controlP5/
 * 
 ********************************************************/
 
import controlP5.*;
import processing.serial.*;
import java.nio.ByteBuffer;

/***********************************************
 * User spcification section
 **********************************************/
int windowWidth = 900;      // set the size of the 
int windowHeight = 600;     // form

float InScaleMin = 0;       // set the Y-Axis Min
float InScaleMax = 1024;    // and Max for both
float OutScaleMin = 0;      // the top and 
float OutScaleMax = 255;    // bottom trends


int windowSpan = 300000;    // number of mS into the past you want to display
int refreshRate = 100;      // how often you want the graph to be reDrawn;

//float displayFactor = 1; //display Time as Milliseconds
//float displayFactor = 1000; //display Time as Seconds
float displayFactor = 60000; //display Time as Minutes

String outputFileName = ""; // if you'd like to output data to 
// a file, specify the path here

/***********************************************
 * end user spec
 **********************************************/

int nextRefresh;
int arrayLength = windowSpan / refreshRate+1;
int[] InputData = new int[arrayLength];     //we might not need them this big, but
int[] SetpointData = new int[arrayLength];  // this is worst case
int[] OutputData = new int[arrayLength];

float inputTop = 25;
float inputHeight = (windowHeight-70)*2/3;
float outputTop = inputHeight+50;
float outputHeight = (windowHeight-70)*1/3;

float ioLeft = 150, ioWidth = windowWidth-ioLeft-50;
float ioRight = ioLeft+ioWidth;
float pointWidth= (ioWidth)/float(arrayLength-1);

int vertCount = 10;

int nPoints = 0;

float Input, Setpoint, Output;

Serial port = null;
int baudrate = 57600;
Integer[] baud_values = {9600, 19200, 38400, 57600, 115200};
int portnumber;

ControlP5 controlP5;
controlP5.Button AMButton, DRButton, PrintMe;
controlP5.Textlabel AMLabel, AMCurrent, InLabel, 
OutLabel, SPLabel, PLabel, 
ILabel, DLabel,DRLabel, DRCurrent;
controlP5.Textfield SPField, InField, OutField, 
PField, IField, DField;
controlP5.DropdownList baud_ddl, ports_ddl;
controlP5.Toggle T;

PrintWriter output;
PFont AxisFont, TitleFont; 
int i;

// State flags
boolean madeContact =false;
boolean justSent = true;

void setup()
{
  frameRate(30);
  size(900 , 600);

  //println(Serial.list());                                                      // * Initialize Serial
  //port = new Serial(this, Serial.list()[0], 57600);                            //   Communication with
  //port.bufferUntil(10);                                                        //   the Arduino

  controlP5 = new ControlP5(this);                                               // * Initialize the various
  //controlP5.addButton("PrintMe").setPosition(10,20).setSize(60,20);            //
  SPField= controlP5.addTextfield("Setpoint",10,100,60,20);                      //   Buttons, Labels, and
  InField = controlP5.addTextfield("Input",10,150,60,20);                        //   Text Fields we'll be
  OutField = controlP5.addTextfield("Output",10,200,60,20);                      //   using
  PField = controlP5.addTextfield("Kp (Proportional)",10,275,60,20);             //
  IField = controlP5.addTextfield("Ki (Integral)",10,325,60,20);                 //
  DField = controlP5.addTextfield("Kd (Derivative)",10,375,60,20);               //
  AMButton = controlP5.addButton("Toggle_AM").setPosition(10,50).setSize(60,20); // ,0.0,10,50,60,20);
  AMLabel = controlP5.addTextlabel("AM","Manual",12,72);                         //
  AMCurrent = controlP5.addTextlabel("AMCurrent","Manual",80,65);                //
  controlP5.addButton("Send_To_Arduino").setPosition(10,475).setSize(126,20);    // ,0.0,10,475,126,20)
  SPLabel=controlP5.addTextlabel("SP","3",80,103);                               //
  InLabel=controlP5.addTextlabel("In","1",80,153);                               //
  OutLabel=controlP5.addTextlabel("Out","2",80,203);                             //
  PLabel=controlP5.addTextlabel("P","4",80,278);                                 //
  ILabel=controlP5.addTextlabel("I","5",80,328);                                 //
  DLabel=controlP5.addTextlabel("D","6",80,378);                                 //
  DRButton = controlP5.addButton("Toggle_DR").setPosition(10,425).setSize(60,20);//  ,0.0,10,425,60,20);
  DRLabel = controlP5.addTextlabel("DR","Direct",12,447);                        //
  DRCurrent = controlP5.addTextlabel("DRCurrent","Direct",80,440);               //
  
  controlP5.addButton("Connect").setPosition(10,525).setSize(126,20);            //  ,0.0,10,525,126,20);
  controlP5.addButton("Disconnect").setPosition(10,550).setSize(126,20);         //  , 0.0,10,550,126,20);
  setup_port_led(10,10);                                                         //
  setup_port_selector(12,500,120,130);                                           //
  
  AxisFont = loadFont("axis.vlw");
  TitleFont = loadFont("Titles.vlw");
 
  nextRefresh=millis();
  if (outputFileName!="") output = createWriter(outputFileName);
}

void draw()
{
  background(200);
  drawGraph();
  drawButtonArea();
  update_port_led();
}

// UI events
void controlEvent(ControlEvent theEvent) {
  if(theEvent.isController()) {
    // baud rate selected
    if(theEvent.getController().getName()=="BAUD") { 
    baudrate = baud_values[(int)theEvent.getController().getValue()];  
    //println("baud="+baudrate); 
  }
    // serial port selected
    if(theEvent.getController().getName()=="PORT") { 
    portnumber = (int)theEvent.getController().getValue(); 
    //println("baud="+Serial.list()[portnumber]); 
  }
  }
}

// port selection functions  .moveTo("Config")
void setup_port_selector(int x, int y, int x2, int y2) {
  baud_ddl = controlP5.addDropdownList("BAUD",x,y,60,140);
  baud_ddl.setItemHeight(18);
  ports_ddl = controlP5.addDropdownList("PORT",x+65,y,60,140);
  ports_ddl.setItemHeight(18);
  baud_ddl.getCaptionLabel().set("BAUD");
  baud_ddl.getCaptionLabel().getStyle().marginTop = 1;
  baud_ddl.setBarHeight(18);
  for (int i=0; i<baud_values.length; i++) baud_ddl.addItem( baud_values[i].toString(),baud_values[i]);
  ports_ddl.getCaptionLabel().set("PORT");
  ports_ddl.getCaptionLabel().getStyle().marginTop = 1;
  ports_ddl.setBarHeight(18);  
  int n_ports = Serial.list().length;
  for(i=0; i<n_ports; i++) ports_ddl.addItem(Serial.list()[i],i);
  baud_ddl.close();
  ports_ddl.close();
}

void setup_port_led(int x, int y) {
  Toggle t = controlP5.addToggle("port_LED",false).setPosition(120,530).setSize(10,10);  // ,false,120,530,10,10);
  t.setLabel("");
  t.setLock(true);
  t.setColorBackground(color(127,0,0));
  t.setColorActive(color(0,255,0));
}

void update_port_led() {
  int c = 0;
  if (port != null) {
    c = port.available();
    controlP5.getController("port_LED").setColorBackground(color(0,127,0));
  }
  if (c > 1) c = 1;
  if ((int)controlP5.getController("port_LED").getValue() != c) controlP5.getController("port_LED").setValue(c);
  if (port == null)  controlP5.getController("port_LED").setColorBackground(color(127,0,0));
}

void drawGraph()
{
  //draw Base, gridlines
  stroke(0);
  fill(230);
  rect(ioLeft+10, inputTop,ioWidth-11 , inputHeight);
  rect(ioLeft+10, outputTop, ioWidth-11, outputHeight);
  stroke(210);

  //Section Titles
  textFont(TitleFont);
  fill(255);
  text("PID Input / Setpoint",(int)ioLeft+10,(int)inputTop-5);
  text("PID Output",(int)ioLeft+10,(int)outputTop-5);


  //GridLines and Titles
  textFont(AxisFont);
  
  //horizontal grid lines
  int interval = (int)inputHeight/5;
  for(int i=0;i<6;i++)
  {
    if(i>0&&i<5) line(ioLeft+1,inputTop+i*interval,ioRight-2,inputTop+i*interval);
    text(str((InScaleMax-InScaleMin)/5*(float)(5-i)+InScaleMin),ioRight+5,inputTop+i*interval+4);

  }
  interval = (int)outputHeight/5;
  for(int i=0;i<6;i++)
  {
    if(i>0&&i<5) line(ioLeft+1,outputTop+i*interval,ioRight-2,outputTop+i*interval);
    text(str((OutScaleMax-OutScaleMin)/5*(float)(5-i)+OutScaleMin),ioRight+5,outputTop+i*interval+4);
  }


  //vertical grid lines and TimeStamps
  int elapsedTime = millis();
  interval = (int)ioWidth/vertCount;
  int shift = elapsedTime*(int)ioWidth / windowSpan;
  shift %=interval;

  int iTimeInterval = windowSpan/vertCount;
  float firstDisplay = (float)(iTimeInterval*(elapsedTime/iTimeInterval))/displayFactor;
  float timeInterval = (float)(iTimeInterval)/displayFactor;
  for(int i=0;i<vertCount;i++)
  {
    int x = (int)ioRight-shift-2-i*interval;

    line(x,inputTop+1,x,inputTop+inputHeight-1);
    line(x,outputTop+1,x,outputTop+outputHeight-1);    

    float t = firstDisplay-(float)i*timeInterval;
    if(t>=0)  text(str(t),x,outputTop+outputHeight+10);
  }


  // add the latest data to the data Arrays.  the values need
  // to be massaged to get them to graph correctly.  they 
  // need to be scaled to fit where they're going, and 
  // because 0, 0 is the top left, we need to flip the values.
  // this is easier than having the user stand on their head
  // to read the graph.
  if(millis() > nextRefresh && madeContact)
  {
    nextRefresh += refreshRate;

    for(int i=nPoints-1;i>0;i--)
    {
      InputData[i]=InputData[i-1];
      SetpointData[i]=SetpointData[i-1];
      OutputData[i]=OutputData[i-1];
    }
    if (nPoints < arrayLength) nPoints++;

    InputData[0] = int(inputHeight)-int(inputHeight*(Input-InScaleMin)/(InScaleMax-InScaleMin));
    SetpointData[0] =int( inputHeight)-int(inputHeight*(Setpoint-InScaleMin)/(InScaleMax-InScaleMin));
    OutputData[0] = int(outputHeight)-int(outputHeight*(Output-OutScaleMin)/(OutScaleMax-OutScaleMin));
  }
  //draw lines for the input, setpoint, and output
  strokeWeight(2);
  for(int i=0; i<nPoints-2; i++)
  {
    int X1 = int(ioRight-2-float(i)*pointWidth);
    int X2 = int(ioRight-2-float(i+1)*pointWidth);
    boolean y1Above, y1Below, y2Above, y2Below;


    //DRAW THE INPUT
    boolean drawLine=true;
    stroke(255,0,0);
    int Y1 = InputData[i];
    int Y2 = InputData[i+1];

    y1Above = (Y1>inputHeight);                     // if both points are outside 
    y1Below = (Y1<0);                               // the min or max, don't draw the 
    y2Above = (Y2>inputHeight);                     // line.  if only one point is 
    y2Below = (Y2<0);                               // outside constrain it to the limit, 
    if(y1Above)                                     // and leave the other one untouched.
    {                                               //
      if(y2Above) drawLine=false;                   //
      else if(y2Below) {                            //
        Y1 = (int)inputHeight;                      //
        Y2 = 0;                                     //
      }                                             //
      else Y1 = (int)inputHeight;                   //
    }                                               //
    else if(y1Below)                                //
    {                                               //
      if(y2Below) drawLine=false;                   //
      else if(y2Above) {                            //
        Y1 = 0;                                     //
        Y2 = (int)inputHeight;                      //
      }                                             //
      else Y1 = 0;                                  //
    }                                               //
    else                                            //
    {                                               //
      if(y2Below) Y2 = 0;                           //
      else if(y2Above) Y2 = (int)inputHeight;       //
    }                                               //

    if(drawLine)
    {
      line(X1,Y1+inputTop, X2, Y2+inputTop);
    }

    //DRAW THE SETPOINT
    drawLine=true;
    stroke(0,255,0);
    Y1 = SetpointData[i];
    Y2 = SetpointData[i+1];

    y1Above = (Y1>(int)inputHeight);                // if both points are outside 
    y1Below = (Y1<0);                               // the min or max, don't draw the 
    y2Above = (Y2>(int)inputHeight);                // line.  if only one point is 
    y2Below = (Y2<0);                               // outside constrain it to the limit, 
    if(y1Above)                                     // and leave the other one untouched.
    {                                               //
      if(y2Above) drawLine=false;                   //
      else if(y2Below) {                            //
        Y1 = (int)(inputHeight);                    //
        Y2 = 0;                                     //
      }                                             //
      else Y1 = (int)(inputHeight);                 //
    }                                               //
    else if(y1Below)                                //
    {                                               //
      if(y2Below) drawLine=false;                   //
      else if(y2Above) {                            //
        Y1 = 0;                                     //
        Y2 = (int)(inputHeight);                    //
      }                                             //
      else Y1 = 0;                                  //
    }                                               //
    else                                            //
    {                                               //
      if(y2Below) Y2 = 0;                           //
      else if(y2Above) Y2 = (int)(inputHeight);     //
    }                                               //

    if(drawLine)
    {
      line(X1, Y1+inputTop, X2, Y2+inputTop);
    }

    //DRAW THE OUTPUT
    drawLine=true;
    stroke(0,0,255);
    Y1 = OutputData[i];
    Y2 = OutputData[i+1];

    y1Above = (Y1>outputHeight);                   // if both points are outside 
    y1Below = (Y1<0);                              // the min or max, don't draw the 
    y2Above = (Y2>outputHeight);                   // line.  if only one point is 
    y2Below = (Y2<0);                              // outside constrain it to the limit, 
    if(y1Above)                                    // and leave the other one untouched.
    {                                              //
      if(y2Above) drawLine=false;                  //
      else if(y2Below) {                           //
        Y1 = (int)outputHeight;                    //
        Y2 = 0;                                    //
      }                                            //
      else Y1 = (int)outputHeight;                 //
    }                                              //
    else if(y1Below)                               //
    {                                              //
      if(y2Below) drawLine=false;                  //
      else if(y2Above) {                           //
        Y1 = 0;                                    //
        Y2 = (int)outputHeight;                    //
      }                                            //  
      else Y1 = 0;                                 //
    }                                              //
    else                                           //
    {                                              //
      if(y2Below) Y2 = 0;                          //
      else if(y2Above) Y2 = (int)outputHeight;     //
    }                                              //

    if(drawLine)
    {
      line(X1, outputTop + Y1, X2, outputTop + Y2);
    }
  }
  strokeWeight(1);
}

void drawButtonArea()
{
  stroke(0);
  fill(100);
  rect(0, 0, ioLeft, windowHeight);
}

//void PrintMe()
//{
//  print("port="+Serial.list()[portnumber]);
//  println(" baud="+baudrate);
//}

void Connect()
{
  //println(Serial.list()[portnumber]);                                           // * Initialize Serial
  //println("baud="+baudrate);
  port = new Serial(this, Serial.list()[portnumber], baudrate);               //   Communication with 
  port.bufferUntil(10);                                           //   the Arduino  
}

void Disconnect()
{
  if (port != null) port.stop(); 
  port = null;
}

void Toggle_AM() {
  if(AMLabel.getValueLabel().getText()=="Manual") 
  {
    AMLabel.setValue("Automatic");
  }
  else
  {
    AMLabel.setValue("Manual");   
  }
}


void Toggle_DR() {
  if(DRLabel.getValueLabel().getText()=="Direct") 
  {
    DRLabel.setValue("Reverse");
  }
  else
  {
    DRLabel.setValue("Direct");   
  }
}

// Sending Floating point values to the arduino
// is a huge pain.  if anyone knows an easier
// way please let know.  the way I'm doing it:
// - Take the 6 floats we need to send and
//   put them in a 6 member float array.
// - using the java ByteBuffer class, convert
//   that array to a 24 member byte array
// - send those bytes to the arduino
void Send_To_Arduino()
{
  float[] toSend = new float[6];

  toSend[0] = float(SPField.getText());
  toSend[1] = float(InField.getText());
  toSend[2] = float(OutField.getText());
  toSend[3] = float(PField.getText());
  toSend[4] = float(IField.getText());
  toSend[5] = float(DField.getText());
  Byte a = (AMLabel.getValueLabel().getText()=="Manual")?(byte)0:(byte)1;
  Byte d = (DRLabel.getValueLabel().getText()=="Direct")?(byte)0:(byte)1;
  port.write(a);
  port.write(d);
  port.write(floatArrayToByteArray(toSend));
  justSent=true;
} 


byte[] floatArrayToByteArray(float[] input)
{
  int len = 4*input.length;
  //int index=0;
  byte[] b = new byte[4];
  byte[] out = new byte[len];
  ByteBuffer buf = ByteBuffer.wrap(b);
  for(int i=0;i<input.length;i++) 
  {
    buf.position(0);
    buf.putFloat(input[i]);
    for(int j=0;j<4;j++) out[j+i*4]=b[3-j];
  }
  return out;
}


//take the string the arduino sends us and parse it
void serialEvent(Serial port)
{
  String read = port.readStringUntil(10);
  if(outputFileName!="") output.print(str(millis())+ " "+read);
  String[] s = split(read, " ");

  if (s.length ==9)
  {
    Setpoint = float(s[1]);           // * pull the information
    Input = float(s[2]);              //   we need out of the
    Output = float(s[3]);             //   string and put it
    SPLabel.setValue(s[1]);           //   where it's needed
    InLabel.setValue(s[2]);           //
    OutLabel.setValue(trim(s[3]));    //
    PLabel.setValue(trim(s[4]));      //
    ILabel.setValue(trim(s[5]));      //
    DLabel.setValue(trim(s[6]));      //
    AMCurrent.setValue(trim(s[7]));   //
    DRCurrent.setValue(trim(s[8]));
    if(justSent)                      // * if this is the first read
    {                                 //   since we sent values to 
      SPField.setText(trim(s[1]));    //   the arduino,  take the
      InField.setText(trim(s[2]));    //   current values and put
      OutField.setText(trim(s[3]));   //   them into the input fields
      PField.setText(trim(s[4]));     //
      IField.setText(trim(s[5]));     //
      DField.setText(trim(s[6]));     //
     // mode = trim(s[7]);              //
      AMLabel.setValue(trim(s[7]));         //
      //dr = trim(s[8]);                //
      DRCurrent.setValue(trim(s[8]));         //
      justSent=false;                 //
    }                                 //

    if(!madeContact) madeContact=true;
  }
}