/*  http://www.instructables.com/id/DIY-Filastruder/
 *   DIY Filament Extruder
 *  http://www.schmalzhaus.com/EasyDriver/Examples/EasyDriverExamples.html
 *   Example 5: Changing motor speed
 *  ==========================================================
 *  Read preset temperature value from pot at A1
 *  Command a MOSFET controlled heating system with PID algorithm
 *  
 * For Reading a 100K thermistor.
 *============================================================
 *
 * (Gnd) ---- (100k-Resistor) -------|------- (100K-Thermistor) ---- (+5v)
 *                                   |
 *                             Analog Pin 0
 *============================================================
 *
 */
 
#include <PID_v1.h>
#include <math.h>
#include <LCD5110_Graph.h>
#include <AccelStepper.h>

LCD5110 myGLCD(8,9,10,12,11);//Nokia 5100 pins  LCD5110(SCK, MOSI, DC, RST, CS);
extern uint8_t start_logo[]; 
//extern uint8_t TinyFont[];
extern uint8_t SmallFont[];
extern uint8_t MediumNumbers[];
//extern uint8_t BigNumbers[];

// Define the stepper and the pins it will use
AccelStepper stepper1(1, 6, 7);   //(1, STEP, DIR);

#define THERMISTOR_PIN 0     //Analog Pin thermistor connected   
#define THRESHOLD_PIN 1      //Analog pin temp pot connected
#define SPEED_PIN 2          //Analog pin temp pot stepper speed
#define HEATER_PIN 5        //digital PWM pin heater connected
#define RIGHT_PIN 2         //right button
#define STOP_PIN  3         //stop button
#define LEFT_PIN  4         //left button

// Define our maximum and minimum speed in steps per second (scale pot to these)
#define  MAX_SPEED 200 //500
#define  MIN_SPEED 0.1

double SetPoint, ActualTemp, Output;

//Specify the links and initial tuning parameters
PID myPID(&ActualTemp, &Output, &SetPoint,2,5,1, DIRECT);

//the time we give the sensor to calibrate (10-30 secs according to the datasheet)
int calibrationTime = 10; 

unsigned long serialTime; //this will help us know when to talk with processing

double Thermister(int RawADC) {
 double Temp;
 Temp = log(((10240000/RawADC) - 10000));
 Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp ))* Temp );
 Temp = Temp - 273.15;            // Convert Kelvin to Celcius
 //Temp = (Temp * 9.0)/ 5.0 + 32.0; // Convert Celcius to Fahrenheit
 return Temp;
}

void setup() {
  pinMode(THRESHOLD_PIN, INPUT);
  //pinMode(SPEED_PIN, INPUT);
  //pinMode(HEATER_PIN, OUTPUT);
  // Set up the three button inputs, with pullups (INPUT_PULLUP)
  pinMode(LEFT_PIN, INPUT_PULLUP);
  pinMode(STOP_PIN, INPUT_PULLUP);
  pinMode(RIGHT_PIN, INPUT_PULLUP);
  
  // The only AccelStepper value we have to set here is the max speeed, which is higher than we'll ever go 
  stepper1.setMaxSpeed(500.0); //10000
  
  Serial.begin(57600);
  // LCD config
  myGLCD.InitLCD();
  myGLCD.clrScr();
  myGLCD.drawBitmap(0, 4, start_logo, 85, 48);  //84, 48
  myGLCD.update();
  delay(5000);
  SetPoint = map(analogRead(THRESHOLD_PIN),0,1023,0,300);
  ActualTemp = double(Thermister(analogRead(THERMISTOR_PIN)));  
  //turn the PID on
  myPID.SetMode(AUTOMATIC);
  //give the sensor some time to calibrate
  //Serial.print("calibrating sensor ");
    for(int i = 0; i < calibrationTime; i++){
      int percent = (70*(i*100/calibrationTime)/100)+5;
      myGLCD.clrScr();
      myGLCD.setFont(SmallFont);
      myGLCD.print("calibrating", 8, 26);
      myGLCD.drawRect(4, 35, 80, 40);
      myGLCD.drawLine(5, 36, percent, 36); // определяет последние строки дисплея, чтобы показать прогрессбар
      myGLCD.drawLine(5, 37, percent, 37); // где 1 начало колонны, 36,37,38 и 39 являются линиями.
      myGLCD.drawLine(5, 38, percent, 38);
      myGLCD.drawLine(5, 39, percent, 39);
      myGLCD.update();
      //Serial.print(".");
      delay(200);
      }
    myGLCD.drawLine(5, 36, 80, 36);
    myGLCD.drawLine(5, 37, 80, 37);
    myGLCD.drawLine(5, 38, 80, 38);
    myGLCD.drawLine(5, 39, 80, 39);
    myGLCD.update();
    //Serial.println(" done");
    //Serial.println("SENSOR ACTIVE");
    delay(200);
}

void loop() {
  static float current_speed = 0.0;         // Holds current motor speed in steps/second
  static int analog_read_counter = 1000;    // Counts down to 0 to fire analog read
  static char sign = 0;                     // Holds -1, 1 or 0 to turn the motor on/off and control direction
  static int analog_value = 0;              // Holds raw analog value.
  SetPoint = map(analogRead(THRESHOLD_PIN),0,1023,0,300);
  ActualTemp = double(Thermister(analogRead(THERMISTOR_PIN)));
  myGLCD.clrScr();
  // If a switch is pushed down (low), set the sign value appropriately
  if (digitalRead(LEFT_PIN) == 0) {
    sign = 1;
  }
  else if (digitalRead(RIGHT_PIN) == 0) {    
    sign = -1;
  }
  else if (digitalRead(STOP_PIN) == 0) {
    sign = 0;
  }

  // We only want to read the pot every so often (because it takes a long time we don't
  // want to do it every time through the main loop).  
  if (analog_read_counter > 0) {
    analog_read_counter--;
  }
  else {
    analog_read_counter = 3000;
    // Now read the pot (from 0 to 1023)
    analog_value = analogRead(SPEED_PIN);
    // Give the stepper a chance to step if it needs to
    stepper1.runSpeed();
    //  And scale the pot's value from min to max speeds
    current_speed = sign * ((analog_value/1023.0) * (MAX_SPEED - MIN_SPEED)) + MIN_SPEED;
    // Update the stepper to run at this new speed
    stepper1.setSpeed(current_speed);
    // Print a message to the LCD.
    myGLCD.setFont(SmallFont);
    myGLCD.print("RPM", 0, 43);
    myGLCD.setFont(MediumNumbers);
    myGLCD.print(String(current_speed), 24, 35);
    myGLCD.setFont(SmallFont);
    myGLCD.print("SET", 0, 10);
    myGLCD.setFont(MediumNumbers);
    myGLCD.print(String(SetPoint), 24, 2);
    myGLCD.setFont(SmallFont);
    myGLCD.print("t.C", 0, 26);
    myGLCD.setFont(MediumNumbers);
    myGLCD.print(String(ActualTemp), 24, 18);
    myGLCD.update();
  }
  myPID.Compute();
  analogWrite(HEATER_PIN,Output);
  stepper1.runSpeed();  // This will run the stepper at a constant speed

  //send-receive with processing if it's time
  if(millis()>serialTime)
  {
    SerialReceive();
    SerialSend();
    serialTime+=500;
  }

 //  Serial.print(SetPoint);  Serial.print("     ");
//  Serial.println(ActualTemp);
}

/********************************************
 * Serial Communication functions / helpers
 ********************************************/


union {                // This Data structure lets
  byte asBytes[24];    // us take the byte array
  float asFloat[6];    // sent from processing and
}                      // easily convert it to a
foo;                   // float array



// getting float values from processing into the arduino
// was no small task.  the way this program does it is
// as follows:
//  * a float takes up 4 bytes.  in processing, convert
//    the array of floats we want to send, into an array
//    of bytes.
//  * send the bytes to the arduino
//  * use a data structure known as a union to convert
//    the array of bytes back into an array of floats

//  the bytes coming from the arduino follow the following
//  format:
//  0: 0=Manual, 1=Auto, else = ? error ?
//  1: 0=Direct, 1=Reverse, else = ? error ?
//  2-5: float setpoint
//  6-9: float input
//  10-13: float output  
//  14-17: float P_Param
//  18-21: float I_Param
//  22-245: float D_Param
void SerialReceive()
{

  // read the bytes sent from Processing
  int index=0;
  byte Auto_Man = -1;
  byte Direct_Reverse = -1;
  while(Serial.available()&&index<26)
  {
    if(index==0) Auto_Man = Serial.read();
    else if(index==1) Direct_Reverse = Serial.read();
    else foo.asBytes[index-2] = Serial.read();
    index++;
  } 
  
  // if the information we got was in the correct format, 
  // read it into the system
  if(index==26  && (Auto_Man==0 || Auto_Man==1)&& (Direct_Reverse==0 || Direct_Reverse==1))
  {
    SetPoint=double(foo.asFloat[0]);
    //Input=double(foo.asFloat[1]);       // * the user has the ability to send the 
                                          //   value of "Input"  in most cases (as 
                                          //   in this one) this is not needed.
    if(Auto_Man==0)                       // * only change the output if we are in 
    {                                     //   manual mode.  otherwise we'll get an
      Output=double(foo.asFloat[2]);      //   output blip, then the controller will 
    }                                     //   overwrite.
    
    double p, i, d;                       // * read in and set the controller tunings
    p = double(foo.asFloat[3]);           //
    i = double(foo.asFloat[4]);           //
    d = double(foo.asFloat[5]);           //
    myPID.SetTunings(p, i, d);            //
    
    if(Auto_Man==0) myPID.SetMode(MANUAL);// * set the controller mode
    else myPID.SetMode(AUTOMATIC);             //
    
    if(Direct_Reverse==0) myPID.SetControllerDirection(DIRECT);// * set the controller Direction
    else myPID.SetControllerDirection(REVERSE);          //
  }
  Serial.flush();                         // * clear any random data from the serial buffer
}

// unlike our tiny microprocessor, the processing ap
// has no problem converting strings into floats, so
// we can just send strings.  much easier than getting
// floats from processing to here no?
void SerialSend()
{
  Serial.print("PID ");
  Serial.print(SetPoint);   
  Serial.print(" ");
  Serial.print(ActualTemp);   
  Serial.print(" ");
  Serial.print(Output);   
  Serial.print(" ");
  Serial.print(myPID.GetKp());   
  Serial.print(" ");
  Serial.print(myPID.GetKi());   
  Serial.print(" ");
  Serial.print(myPID.GetKd());   
  Serial.print(" ");
  if(myPID.GetMode()==AUTOMATIC) Serial.print("Automatic");
  else Serial.print("Manual");  
  Serial.print(" ");
  if(myPID.GetDirection()==DIRECT) Serial.println("Direct");
  else Serial.println("Reverse");
}

